import socket

def cidr(address):
    try:
        x = socket.inet_aton(address)
    except:
        raise ValueError("%s is not a valid ip address" % address)
    x = (ord(x[0])<<24) + (ord(x[1])<<16) + (ord(x[2])<<8) + ord(x[3])
    return x

def in_cidr(address, net):
    """
    Takes two strings, the first an IP address, the second a network in
    CIDR format
    Returns True if the adress is in the network

    Example:
    >>> in_cidr('10.0.0.7', '10.0.0.0/29')
    True
    >>> in_cidr('10.0.0.8', '10.0.0.0/29')
    False
    >>> in_cidr('127.0.0.1', '127.0.0.1')
    True
    """

    mask = 0xFFFFFFFF
    components = net.split("/")
    if len(components) > 1:
        mask = ~(0xFFFFFFFF >> int(components[1]))

    x = cidr(address)
    y = cidr(components[0])

    return (x & mask) == y

#if __name__ == '__main__':
#    print in_cidr('10.0.0.7', '10.0.0.0/29')
#    print in_cidr('10.0.0.8', '10.0.0.0/29')
#    print in_cidr('127.0.0.1', '127.0.0.1')
#    print in_cidr('10.0.0.20', '10.0.0.0/24')



from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import classImplements
from Products.PluggableAuthService.interfaces.plugins import IAuthenticationPlugin, IExtractionPlugin

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import cidr

manage_addIPAuthPluginForm = PageTemplateFile(
            'www/ipAuthAdd', globals(), __name__='manage_addIPAuthPluginForm' )

def manage_addIPAuthPlugin(dispatcher, id, title=None, REQUEST=None):
    """ Add a AuthIPPlugin to a Pluggable Auth Service. """

    obj = AuthIPPlugin(id, title)
    dispatcher._setObject(obj.getId(), obj)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( '%s/manage_workspace?manage_tabs_message=IPAuthPlugin+added.' % dispatcher.absolute_url())

class AuthIPPlugin(BasePlugin):
    """ PAS plugin for using IP Address to log in. """

    meta_type = 'AuthIPPlugin'

    security = ClassSecurityInfo()

    _properties = (
        {'id':'remote_ip_addresses', 'type':'text', 'mode':'w','label':'A list of <IP Addresses:login id>'},)

    remote_ip_addresses=''

    def __init__(self, id, title=None):
        self._id = self.id = id
        self.title = title

    #
    #   IAuthenticationPlugin implementation (PluggableAuthService interface)
    #
    security.declarePrivate('authenticateCredentials')
    def authenticateCredentials(self, credentials):
        """ See IAuthenticationPlugin.

        o We expect the credentials to be those returned by
          ILoginPasswordHostExtractionPlugin.
        """
        login = credentials.get('loginid')
        return login, login

    security.declarePrivate('extractCredentials')
    def extractCredentials(self, request):
        """Extract credentials for ip address"""

        clean_rem_addresses=[]
        for s in self.remote_ip_addresses.split("\n"):
            s.strip(" ")
            if s == '':
                continue
            s = s.split(":")
            clean_rem_addresses.append(tuple(s))
        
        forwarded_ips=[]
        for ip_address in request.get('HTTP_X_FORWARDED_FOR', '').split(','):
            ip_address = ip_address.strip()
            if ip_address:
                forwarded_ips.append(ip_address)

        for tup in clean_rem_addresses:
            if len(tup) <= 1:
                return {}
            clientAddr = request.getClientAddr()
            if cidr.in_cidr(clientAddr, tup[0]):
                return {'loginid':tup[1], 'remote_address':clientAddr,}
            for ip_address in forwarded_ips:
                if cidr.in_cidr(ip_address, tup[0]):
                    return {'loginid':tup[1], 'remote_address':ip_address,}
        return {}

classImplements(AuthIPPlugin, IAuthenticationPlugin, IExtractionPlugin)

InitializeClass(AuthIPPlugin)



"""
"""

from AccessControl.Permissions import add_user_folders
from Products.PluggableAuthService.PluggableAuthService import registerMultiPlugin
from AuthIPPlugin import AuthIPPlugin, manage_addIPAuthPlugin, manage_addIPAuthPluginForm

_initialized = False

def initialize(context):
    """Initialize IPAuthPlugin"""
    # This is sometimes called twice, protect against that.
    global _initialized
    if not _initialized:
        registerMultiPlugin(AuthIPPlugin.meta_type)

        context.registerClass( AuthIPPlugin,
                                permission=add_user_folders,
                                constructors=(manage_addIPAuthPluginForm,manage_addIPAuthPlugin),
                                icon='www/PluggableAuthService.png',
                                visibility=None,
                            )
        _initialized = True



import os
from setuptools import setup, find_packages

setup(name='Products.PASIPAuth',
      version='0.2',
      package_data = {'Products.PASIPAuth': ['www/*', 'configure.zcml']},
      description='PAS plugin that authenticates by ip address',
      long_description=open("README.txt").read(),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Programming Language :: Python",
        "Intended Audience :: System Administrators",
        "Topic :: System :: Systems Administration :: Authentication/Directory",
        "Topic :: Software Development :: Libraries :: Python Modules",
      ],
      author='Izak Burger',
      author_email='izak@upfrontsystems.co.za',
      url='http://www.upfrontsystems.co.za/',
      license='GPL',
      packages=find_packages(),
      namespace_packages=['Products'],
)
